"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const clime_1 = require("clime");
const device_1 = require("../../../options/device");
const impl_1 = require("@iotize/device-client.js/protocol/impl");
const mcu_flasher_js_1 = require("@iotize/mcu-flasher.js");
const file_exists_1 = require("../../../validators/file-exists");
const ini_1 = require("ini");
const fs_1 = require("fs");
const util_1 = require("../../../../tools/util");
const config_1 = require("../../../../config");
const inversify_config_1 = require("../../../../inversify.config");
const types_1 = require("../../../../types");
const update_1 = require("../../../commands/device/firmware/update");
class MyOptions extends device_1.DeviceOptions {
}
__decorate([
    clime_1.option({
        description: 'TapNLink firmware version',
        required: false,
        default: 'latest'
    }),
    __metadata("design:type", String)
], MyOptions.prototype, "version", void 0);
__decorate([
    clime_1.option({
        description: 'Skip TapNLink update',
        toggle: true,
        default: false
    }),
    __metadata("design:type", Boolean)
], MyOptions.prototype, "skipTapnlink", void 0);
__decorate([
    clime_1.option({
        description: 'Force (do not ask for confirmation)',
        toggle: true,
        default: false
    }),
    __metadata("design:type", Boolean)
], MyOptions.prototype, "force", void 0);
__decorate([
    clime_1.option({
        description: 'Chunk size of single packet part',
        flag: 's'
    }),
    __metadata("design:type", Number)
], MyOptions.prototype, "chunkSize", void 0);
__decorate([
    clime_1.option({
        description: 'Perform a factory reset before applying update',
        toggle: true,
        default: false
    }),
    __metadata("design:type", Boolean)
], MyOptions.prototype, "factoryReset", void 0);
__decorate([
    clime_1.option({
        description: 'Firmware update file path',
        validator: new file_exists_1.FileExists(),
        required: true
    }),
    __metadata("design:type", String)
], MyOptions.prototype, "file", void 0);
__decorate([
    clime_1.option({
        description: 'Firmware update provider',
        default: update_1.FirmwareUpdateProviderType.s3,
        flag: 'p'
    }),
    __metadata("design:type", String)
], MyOptions.prototype, "firmwareUpdateProvider", void 0);
exports.MyOptions = MyOptions;
let default_1 = class default_1 extends clime_1.Command {
    constructor() {
        super(...arguments);
        this.tapnlinkUpdater = inversify_config_1.container.get(types_1.TYPES.TapNLinkUpdater);
    }
    execute(options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // if (!options.credentials) {
                //     options.credentials = 'admin:admin'
                // }
                let tap = yield options.getDevice();
                if (tap.protocol instanceof impl_1.AbstractComProtocol) {
                    tap.protocol.options.connect.timeout = 10000;
                    tap.protocol.options.send.timeout = 10000;
                }
                let sn = (yield tap.service.device.getSerialNumber()).body();
                // let currentVersion = (await tap.service.device.getFirmwareVersion()).body()!
                // let adpStatus = await (tap.service as any)._adp.getAdpStatus()
                global.output.infonl(`Serial number: ${sn}`);
                // global.output.infonl(`Tap firmware version: ${currentVersion}`)
                // if (!adpStatus.isSuccessful()) {
                //     throw new Error(`This is not an adp device (${new ResponseError(adpStatus).message})`)
                // }
                global.output.infonl(`===== Start update step 1/2 =====`);
                yield this._updateTapNPass(tap, options);
                global.output.success(` ===== Update step 1/2 successful =====`);
                global.output.infonl(`===== Start update step 2/2 =====`);
                if (!options.skipTapnlink) {
                    yield this._updateTapNLink(tap, options);
                }
                else {
                    global.output.info(`Skip`);
                }
                global.output.success(`===== Update step 2/2 successful =====`);
                global.output.success(`Update successful.`);
                process.exit(0);
            }
            catch (err) {
                this._onError(err);
                process.exit(err.code || 1);
            }
        });
    }
    _updateTapNLink(tap, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tapnlinkUpdater.update(tap, options);
        });
    }
    _updateTapNPass(tap, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // Adp sim file
            let simFileContent = yield util_1.getContent(config_1.SIM_FILE_ADDRESS);
            // console.info("SIM FILE => ", simFileContent);
            let simFile = ini_1.decode(simFileContent);
            // console.info("SIM FILE => ", simFile);
            let pluginFilename = simFile.FLASH.PluginHexFileName;
            if (!pluginFilename) {
                throw new Error(`Missing plugin file name in configuration file ${config_1.SIM_FILE_ADDRESS}`);
            }
            let pluginString = yield util_1.getContent(`https://raw.githubusercontent.com/iotize-sas/simfiles/master/Plugins/${pluginFilename}`); // TODO config
            // console.info("pluginString => ", pluginString);
            let mcuDeveloper = mcu_flasher_js_1.MicroControllerDeveloper.create(tap, simFileContent, pluginString);
            let programString = fs_1.readFileSync(options.file).toString();
            // console.info("programString => ", programString);
            let progressBar;
            let updater = inversify_config_1.container.get(types_1.TYPES.TapNPassUpdater);
            let lastMessage = undefined;
            let lastProgress = undefined;
            let obs = updater.update(mcuDeveloper, programString);
            obs
                .subscribe({
                next: (progress) => {
                    // console.info(`PROGRESS:`, progress)
                    if (lastMessage != progress.message) {
                        lastMessage = progress.message;
                        if (progressBar) {
                            progressBar.terminate();
                            progressBar = undefined;
                        }
                        global.output.infonl(`${progress.type} ${progress.message}`);
                        switch (progress.type) {
                            case 'read':
                                break;
                            case 'installRPP':
                                break;
                            case 'erase':
                                break;
                            case 'blankCheck':
                                break;
                            case 'program':
                                break;
                            case 'directProcedure':
                                break;
                            case 'readMemMult':
                                break;
                        }
                    }
                    if (progress.data && progress.data.stepIndex) {
                        let progressInfo = progress.data;
                        if (!lastProgress || (lastProgress.stepName != progressInfo.stepName)) {
                            // console.info(progressInfo);
                            progressBar = global.output.progressBar(`${progressInfo.stepName} [:bar] :rate/bps :percent :etas :current/:total bytes`, {
                                total: progressInfo.endIndex
                            });
                        }
                        if (progressBar) {
                            // progressBar.curr = progressInfo.stepIndex;
                            progressBar.update(progressInfo.percentageState / 100);
                        }
                        lastProgress = progressInfo;
                    }
                },
                error: this._onError,
                complete: () => {
                    if (progressBar) {
                        progressBar.terminate();
                    }
                }
            });
            yield obs.toPromise();
        });
    }
    _onError(err) {
        console.log(`Error:`, err);
        global.output.error(err.message);
        process.exit(1);
    }
    _checkErrorCode(name, code) {
        if (code != mcu_flasher_js_1.ReturnCode.DNG_ERR_OK) {
            throw new Error(`Command failed ${name} with error code ${code} (${mcu_flasher_js_1.ReturnCode[code]})`);
        }
    }
};
__decorate([
    clime_1.metadata,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [MyOptions]),
    __metadata("design:returntype", Promise)
], default_1.prototype, "execute", null);
default_1 = __decorate([
    clime_1.command({
        description: 'Update TapNPass firmware',
    })
], default_1);
exports.default = default_1;
//# sourceMappingURL=update.js.map