"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const clime_1 = require("clime");
const device_1 = require("../../../options/device");
const inversify_config_1 = require("../../../../inversify.config");
const types_1 = require("../../../../types");
const in_list_1 = require("../../../validators/in-list");
var FirmwareUpdateProviderType;
(function (FirmwareUpdateProviderType) {
    FirmwareUpdateProviderType["s3"] = "s3";
    FirmwareUpdateProviderType["server"] = "server";
})(FirmwareUpdateProviderType = exports.FirmwareUpdateProviderType || (exports.FirmwareUpdateProviderType = {}));
class TapFirmwareUpdateOptions extends device_1.DeviceOptions {
}
__decorate([
    clime_1.option({
        description: 'Version',
        required: false,
        default: 'latest'
    }),
    __metadata("design:type", String)
], TapFirmwareUpdateOptions.prototype, "version", void 0);
__decorate([
    clime_1.option({
        description: 'Force (do not ask for confirmation)',
        toggle: true,
        default: false
    }),
    __metadata("design:type", Boolean)
], TapFirmwareUpdateOptions.prototype, "force", void 0);
__decorate([
    clime_1.option({
        description: 'Chunk size of single packet part',
        flag: 's'
    }),
    __metadata("design:type", Number)
], TapFirmwareUpdateOptions.prototype, "chunkSize", void 0);
__decorate([
    clime_1.option({
        description: 'Perform a factory reset before applying update',
        toggle: true,
        default: false
    }),
    __metadata("design:type", Boolean)
], TapFirmwareUpdateOptions.prototype, "factoryReset", void 0);
__decorate([
    clime_1.option({
        description: 'Firmware update provider',
        default: FirmwareUpdateProviderType.s3,
        flag: 'p',
        validator: new in_list_1.InListValidator(Object.keys(FirmwareUpdateProviderType))
    }),
    __metadata("design:type", String)
], TapFirmwareUpdateOptions.prototype, "firmwareUpdateProvider", void 0);
exports.TapFirmwareUpdateOptions = TapFirmwareUpdateOptions;
let default_1 = class default_1 extends clime_1.Command {
    execute(options) {
        return __awaiter(this, void 0, void 0, function* () {
            switch (options.firmwareUpdateProvider) {
                case FirmwareUpdateProviderType.s3:
                    inversify_config_1.container.rebind(types_1.TYPES.FirmwareUpdateProvider).toDynamicValue(() => {
                        return inversify_config_1.container.get(types_1.TYPES.S3FirmwareUpdateProvider);
                    });
                    break;
                case FirmwareUpdateProviderType.server:
                    inversify_config_1.container.rebind(types_1.TYPES.FirmwareUpdateProvider).toDynamicValue(() => {
                        return inversify_config_1.container.get(types_1.TYPES.RestFirmwareUpdateProvider);
                    });
                    break;
            }
            let tapnlinkUpdater = inversify_config_1.container.get(types_1.TYPES.TapNLinkUpdater);
            if (!options.credentials) {
                options.credentials = 'admin:admin'; // TODO config
            }
            try {
                let tap = yield options.getDevice();
                let sn = (yield tap.service.device.getSerialNumber()).body();
                global.output.infonl(`Serial number: ${sn}`);
                yield tapnlinkUpdater.update(tap, options);
                this._onUpdateSuccessful();
            }
            catch (err) {
                // Depending of the firmware updater version, a timeout may occure when execution single packet 
                // However this is not an error
                if (err.name === 'TimeoutError') {
                    this._onUpdateSuccessful();
                }
                else {
                    console.log(err);
                    global.output.error(`Applying update failed: ${err.message}`);
                    process.exit(1);
                }
            }
        });
    }
    _onUpdateSuccessful() {
        global.output.success(`Update done!`);
        process.exit(0);
    }
};
__decorate([
    clime_1.metadata,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [TapFirmwareUpdateOptions]),
    __metadata("design:returntype", Promise)
], default_1.prototype, "execute", null);
default_1 = __decorate([
    clime_1.command({
        description: 'TapNPass firmware update'
    })
], default_1);
exports.default = default_1;
//# sourceMappingURL=update.js.map