"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const clime_1 = require("clime");
const protocol_factory_1 = require("../../tools/protocol-factory");
const relay_server_factory_1 = require("../../tools/relay/relay-server-factory");
const impl_1 = require("@iotize/device-client.js/relay/impl");
const constants_1 = require("../constants");
const option_helper_1 = require("../util/option-helper");
const impl_2 = require("@iotize/device-client.js/protocol/impl");
const relay_1 = require("@iotize/device-com-socket.node/dist/relay");
class RelayOptions extends clime_1.Options {
}
__decorate([
    clime_1.option({
        description: 'Source protocol options (where commands are comming from. For example: tcp://localhost:2000 for socket or ws://localhost:2000 for websocket)',
        required: false,
        default: global.getConfig('default.relay.protocol', constants_1.Constants.DEFAULT_RELAY_URL),
        flag: "p"
    }),
    __metadata("design:type", String)
], RelayOptions.prototype, "protocol", void 0);
__decorate([
    clime_1.option({
        description: 'Local',
        required: false,
        toggle: true,
        default: false
    }),
    __metadata("design:type", Boolean)
], RelayOptions.prototype, "local", void 0);
__decorate([
    clime_1.option(option_helper_1.OptionHelper.OPTION_DEVICE),
    __metadata("design:type", String)
], RelayOptions.prototype, "device", void 0);
__decorate([
    clime_1.option({
        description: 'Create server (socket or websocket)',
        required: false,
        default: global.getConfig('default.relay.createServer', false),
        toggle: true,
        flag: "c",
        name: 'create-server'
    }),
    __metadata("design:type", Boolean)
], RelayOptions.prototype, "createServer", void 0);
__decorate([
    clime_1.option({
        description: 'Socket server port',
        required: false,
        default: global.getConfig('default.relay.server-port', constants_1.Constants.DEFAULT_SERVER_PORT),
    }),
    __metadata("design:type", Number)
], RelayOptions.prototype, "socketPort", void 0);
__decorate([
    clime_1.option({
        description: 'Websocket server port',
        required: false,
        default: 2080,
    }),
    __metadata("design:type", Number)
], RelayOptions.prototype, "websocketPort", void 0);
__decorate([
    clime_1.option({
        description: 'Time (in milliseconds) before a timeout error is thrown while waiting for relay to start',
        required: false,
        default: -1
    }),
    __metadata("design:type", Number)
], RelayOptions.prototype, "timeout", void 0);
__decorate([
    clime_1.option({
        description: 'Kill process after duration (in milliseconds)',
        required: false,
    }),
    __metadata("design:type", Number)
], RelayOptions.prototype, "duration", void 0);
__decorate([
    clime_1.option({
        description: 'Auto restart',
        required: false,
        toggle: true,
        flag: 'a',
        default: false
    }),
    __metadata("design:type", Number)
], RelayOptions.prototype, "autoRestart", void 0);
exports.RelayOptions = RelayOptions;
/**
 * iode relay --device XXX
 * iode relay --socket
 * iode relay --websocket
 */
let default_1 = class default_1 extends clime_1.Command {
    /**
     * iode relay --device XXX
     * iode relay --socket
     * iode relay --websocket
     */
    constructor() {
        super(...arguments);
        this.relays = [];
        this.servers = [];
    }
    onExit() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let relay of this.relays) {
                yield relay.stop();
            }
            for (let server of this.servers) {
                yield server.stop();
            }
            process.exit(0);
        });
    }
    // protected client!: TypedIoTizeClient;
    execute(options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (options.autoRestart) {
                throw new Error(`Auto restart not implemented yet`);
            }
            process.on('SIGINT', () => {
                console.log("Caught interrupt signal");
                this.onExit();
            });
            if (options.duration) {
                setTimeout(() => {
                    console.info(`Exit after expected duration of ${options.duration} milliseconds`);
                    this.onExit();
                }, options.duration);
            }
            let targetProtocol = yield this.createTargetProtocol(options);
            if (targetProtocol instanceof impl_2.QueueComProtocol) {
                targetProtocol.options.send.timeout = 2000;
            }
            let promise;
            if (options.local) {
                let webSocketServerOptions = {
                    port: options.websocketPort
                };
                let webSocketProtocol = yield this.createSourceProtocol(`ws://localhost:${webSocketServerOptions.port}`);
                let socketServerOptions = {
                    port: options.socketPort
                };
                let socketServer = new impl_1.RelayServer(new relay_1.SocketAdapter.Server(), socketServerOptions);
                this.servers.push(socketServer);
                let webSocketServer = relay_server_factory_1.RelayServerFactory.fromType('websocket', webSocketServerOptions);
                this.servers.push(webSocketServer);
                yield socketServer.start();
                global.output.success(`Socket server is running on ${socketServerOptions.port}...`);
                yield webSocketServer.start();
                global.output.success(`Web Socket server is running on ${webSocketServerOptions.port}`);
                let socketRelay = new impl_1.ComRelay();
                socketRelay.setTargetProtocol(targetProtocol);
                socketRelay.setSourceProtocol(yield this.createSourceProtocol(`tcp://localhost:${socketServerOptions.port}}`));
                this.relays.push(socketRelay);
                let webSocketRelay = new impl_1.ComRelay();
                webSocketRelay.setTargetProtocol(targetProtocol);
                webSocketRelay.setSourceProtocol(webSocketProtocol);
                this.relays.push(webSocketRelay);
                promise =
                    Promise.all([
                        webSocketRelay.start({
                            timeout: options.timeout
                        }),
                        socketRelay.start({
                            timeout: options.timeout
                        }),
                    ]);
            }
            else {
                let relayServer;
                let relay;
                let sourceProtocol = yield this.createSourceProtocol(options.protocol);
                if (options.createServer) {
                    relayServer = this.createServer(sourceProtocol, options);
                    yield relayServer.start();
                    global.output.success(`Relay server is running on ${options.socketPort}...`);
                    this.servers.push(relayServer);
                }
                relay = new impl_1.ComRelay();
                relay.setTargetProtocol(targetProtocol);
                relay.setSourceProtocol(sourceProtocol);
                this.relays.push(relay);
                promise = relay.start({
                    timeout: options.timeout
                });
                // if (options.background){
                //     promise
                //         .catch((err) => {
                //             console.error(err);
                //         });
                //     return;
                // }
                // else{
            }
            return promise;
            // }
        });
    }
    createServer(sourceProtocol, options) {
        return relay_server_factory_1.RelayServerFactory.fromProtocol(sourceProtocol, {
            port: options.socketPort
        });
    }
    createTargetProtocol(options) {
        return protocol_factory_1.ProtocolFactory.fromUrl(options.device);
    }
    createSourceProtocol(url) {
        return protocol_factory_1.ProtocolFactory.fromUrl(url);
    }
};
__decorate([
    clime_1.metadata,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [RelayOptions]),
    __metadata("design:returntype", Promise)
], default_1.prototype, "execute", null);
default_1 = __decorate([
    clime_1.command({
        description: 'Start an IoTize device communication relay.  ',
    })
], default_1);
exports.default = default_1;
//# sourceMappingURL=relay.js.map