#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const clime_1 = require("clime");
require("./setup-global");
require("./configure-log");
const fs_1 = require("fs");
function addIfExist(commands, info) {
    if (fs_1.existsSync(info.path)) {
        commands.push(info);
    }
}
// Global exposure
global.appName = 'iode';
// in App.test.tsx
const WebSocket = require('ws');
global.WebSocket = WebSocket;
// The second parameter is the path to folder that contains command modules.
let commandsDirectory = path_1.resolve(__dirname, 'commands');
let commands = [
    {
        label: 'Build in',
        path: commandsDirectory
    },
    {
        label: 'Tools',
        path: path_1.resolve(__dirname, 'commands-tools')
    },
    {
        label: 'TapNPass',
        path: path_1.resolve(__dirname, 'commands-tapnpass')
    },
];
addIfExist(commands, {
    label: 'Cloud',
    path: path_1.resolve(__dirname, 'commands-firebase')
});
addIfExist(commands, {
    label: 'Experimental',
    path: path_1.resolve(__dirname, 'commands-experimental')
});
addIfExist(commands, {
    label: 'Gateway',
    path: path_1.resolve(__dirname, 'commands-gateway'),
});
addIfExist(commands, {
    label: 'Cayenne',
    path: path_1.resolve(__dirname, 'commands-cayenne'),
});
let cli = new clime_1.CLI(global.appName, commands);
// Clime in its core provides an object-based command-line infrastructure.
// To have it work as a common CLI, a shim needs to be applied:
let shim = new clime_1.Shim(cli);
try {
    let result = shim.execute(process.argv);
    console.log(`execute result type ${typeof result}`);
    if (result instanceof Promise) {
        result
            .then(() => {
            require('./teardown');
        })
            .catch((err) => {
            console.warn(`Error while running command:`, err);
            global.output.error(`${err.message}`);
        });
    }
    else {
        require('./teardown');
    }
}
catch (err) {
    console.log(err);
    global.output.error(`${err.message}`);
}
//# sourceMappingURL=main.js.map