"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const default_1 = require("./default");
const clime_1 = require("clime");
const option_helper_1 = require("../util/option-helper");
const device_factory_1 = require("../../tools/device-factory");
const regex_validator_1 = require("../validators/regex-validator");
const file_exists_1 = require("../validators/file-exists");
const iotize_config_factory_1 = require("@iotize/device-config.js/iotize-config-factory");
const file_1 = require("../../tools/file");
const tap_config_configurator_1 = require("@iotize/device-extras.js/configurator/tap-config-configurator");
const fs_1 = require("fs");
const jsyaml = require("js-yaml");
const device_admin_js_1 = require("@iotize/device-admin.js");
class DeviceOptions extends default_1.DefaultOptions {
    getDevice(connect = true, loadConfigIfExists = true, adminRequired = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let tap = yield device_factory_1.DeviceFactory.fromUrl(this.device, connect);
            if (loadConfigIfExists) {
                if (this.config) {
                    let configurator = new tap_config_configurator_1.TapConfigConfigurator(this.getConfigModel());
                    yield tap.configure(configurator);
                }
            }
            // Flush
            // await tap.protocol.send(Uint8Array.from([0]));
            if (this.secure) {
                global.output.infonl(`Setting secure communication required...\n`);
                yield tap.enableEncryption(true);
            }
            if (this.credentials) {
                yield option_helper_1.OptionHelper.logFromOptions(tap, this);
            }
            return tap;
        });
    }
    getConfigModel() {
        let config = file_1.FileHelper.getContent(this.config, 'string');
        return iotize_config_factory_1.IotizeConfigFactory.fromXML(config);
    }
    getApiExplorer() {
        if (!this.spec) {
            return undefined;
        }
        let filename = this.spec;
        let api = device_admin_js_1.OpenApiFactory.create(jsyaml.load(fs_1.readFileSync(filename).toString()));
        return api;
    }
}
__decorate([
    clime_1.option(option_helper_1.OptionHelper.OPTION_DEVICE),
    __metadata("design:type", String)
], DeviceOptions.prototype, "device", void 0);
__decorate([
    clime_1.option({
        flag: 'c',
        description: 'Tap device configuration file path',
        default: global.getConfig('default.config'),
        validator: new file_exists_1.FileExists()
    }),
    __metadata("design:type", String)
], DeviceOptions.prototype, "config", void 0);
__decorate([
    clime_1.option({
        flag: 's',
        description: 'Spec file',
        default: global.getConfig('default.spec'),
        validator: new file_exists_1.FileExists(),
        required: false
    }),
    __metadata("design:type", String)
], DeviceOptions.prototype, "spec", void 0);
__decorate([
    clime_1.option({
        description: 'Credentials formatted as username:password (it log you in the selected device)',
        required: false,
        validator: new regex_validator_1.RegexValidator(/.*:.*/)
    }),
    __metadata("design:type", String)
], DeviceOptions.prototype, "credentials", void 0);
__decorate([
    clime_1.option({
        description: 'Use secure communication (enabe encryption for commands and responses)',
        default: global.getConfig("default.secure"),
        toggle: true,
    }),
    __metadata("design:type", Boolean)
], DeviceOptions.prototype, "secure", void 0);
exports.DeviceOptions = DeviceOptions;
//# sourceMappingURL=device.js.map