"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_provider_1 = require("./../lib/config/config-provider");
const input_1 = require("../lib/input");
const output_1 = require("../lib/output");
const missing_config_key_error_1 = require("../lib/config/missing-config-key-error");
const clime_1 = require("clime");
const path_1 = require("path");
global.input = new input_1.Input();
global.output = new output_1.Output();
if (!process.env.APPDATA) {
    throw new Error("No APPDATA path defined... You must define environment variable APPDATA");
}
let configPath = path_1.resolve(process.env.APPDATA, "iodecli");
global.configProvider = new config_provider_1.ConfigProvider({
    basePath: configPath
});
global.configProvider.setProfile("default");
global.getConfig = (key, defaultValue) => {
    try {
        return global.configProvider.config().get(key, defaultValue);
    }
    catch (err) {
        if (err instanceof missing_config_key_error_1.MissingConfigKeyError) {
            if (defaultValue === undefined) {
                return defaultValue;
            }
        }
    }
};
global.requireConfig = (key, defaultValue) => {
    try {
        return global.configProvider.config().get(key, defaultValue);
    }
    catch (err) {
        if (err instanceof missing_config_key_error_1.MissingConfigKeyError) {
            throw new clime_1.ExpectedError(err.message);
        }
    }
};
console.log(`Configuration file: ${configPath}`);
//# sourceMappingURL=setup-global.js.map