"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const types_1 = require("./types");
const device_scanner_1 = require("./tools/scanner/device-scanner");
const lib_1 = require("@iotize/device-com-ble.node/lib");
const testing_grammar_js_1 = require("@iotize/testing-grammar.js");
const format_helper_1 = require("@iotize/device-client.js/core/format/format-helper");
const body_1 = require("@iotize/device-client.js/client/impl/converter/body");
const firwmare_update_provider_1 = require("./lib/firwmare-update-provider");
const tapnlink_update_service_1 = require("./lib/tapnlink-admin/tapnlink-update.service");
const tapnpass_update_service_1 = require("./lib/tapnpass-admin/tapnpass-update.service");
// import { AppConfig } from "@iotize-private/firebase-hub.js";
const device_admin_js_1 = require("@iotize/device-admin.js");
const container = new inversify_1.Container();
exports.container = container;
;
function pluginExists(path) {
    try {
        require.resolve(path);
        return true;
    }
    catch (err) {
        return false;
    }
}
container.bind(types_1.TYPES.TapGatewayProvider).toProvider((context) => {
    return (type, device, options) => {
        try {
            let relativePath = `./plugins/${type}`;
            let plugin;
            if (pluginExists(relativePath)) {
                console.log(`Loading local plugin ${relativePath}`);
                plugin = require(`./plugins/${type}`).default;
            }
            else {
                console.log(`Loading lib plugin ${type}`);
                plugin = require(`${type}`);
            }
            if (typeof plugin.tapGateway != 'function') {
                throw new Error(`Invalid plugin definition "${type}". It should export function tapGateway`);
            }
            return plugin.tapGateway(device, options);
        }
        catch (err) {
            console.error('Cannot load tap gateway provider', type, err);
            return Promise.reject(err);
        }
    };
});
container.bind(types_1.TYPES.NobleProvider).toProvider((context) => {
    return () => {
        let useUwp = ('NOBLE_UWP' in process.env) && (process.env['NOBLE_UWP'] != 'false');
        console.log('NOBLE_UWP SET: ', useUwp);
        return lib_1.NobleHelper
            .getNoble(useUwp)
            .then((noble) => {
            return lib_1.NobleHelper.waitForNobleState('poweredOn', 5000, noble)
                .then(() => {
                return noble;
            });
        });
    };
});
container.bind(types_1.TYPES.DeviceScanner).to(device_scanner_1.DeviceScanner);
container.bind(types_1.TYPES.DeviceScannerProvider).toProvider((context) => {
    return () => {
        return context.container
            .get(types_1.TYPES.NobleProvider)()
            .then((noble) => {
            if (noble.uwp) {
                console.info(`USING NOBLE UWP`);
            }
            return new device_scanner_1.DeviceScanner(new lib_1.NobleBleScanner(noble));
        });
    };
});
container.bind(types_1.TYPES.ScriptRunnerProvider).toDynamicValue((context) => {
    return () => {
        return new testing_grammar_js_1.ScriptRunner({
            display: (level, message) => {
                global.output.infonl(`${message}`);
            },
            output: (response, wordSize) => {
                if (!response) {
                    return;
                }
                let message;
                if (!wordSize) {
                    message = format_helper_1.FormatHelper.toAsciiString(response.rawBody());
                }
                else {
                    message = body_1.NumberConverter.fromOpaqueMsb(response.rawBody(), wordSize).toString();
                }
                global.output.infonl(`${message}`);
            }
        });
    };
});
container.bind(types_1.TYPES.ScriptRunnerProvider).toDynamicValue((context) => {
    return () => {
        return new testing_grammar_js_1.ScriptRunner({
            display: (level, message) => {
                global.output.infonl(`${message}`);
            },
            output: (response, wordSize) => {
                if (!response) {
                    return;
                }
                let message;
                if (!wordSize) {
                    message = format_helper_1.FormatHelper.toAsciiString(response.rawBody());
                }
                else {
                    message = body_1.NumberConverter.fromOpaqueMsb(response.rawBody(), wordSize).toString();
                }
                global.output.infonl(`${message}`);
            }
        });
    };
});
let profileConfig = global.configProvider.config();
container.bind(types_1.TYPES.RestFirmwareUpdateProvider).toDynamicValue((context) => {
    return new firwmare_update_provider_1.RestFirmwareUpdateProvider({
        baseURL: profileConfig.get('firmwareUpdateServer.url', 'http://localhost:3010/'),
        timeout: profileConfig.get('firmwareUpdateServer.timeout', 10000),
        headers: {}
    });
});
container.bind(types_1.TYPES.S3FirmwareUpdateProvider).toDynamicValue((context) => {
    return new firwmare_update_provider_1.S3FirmwareUpdateProvider({
        bucket: 'prod.iotize.tap.data',
        region: 'eu-west-1'
    });
});
container.bind(types_1.TYPES.SchemaTapConfigurator).toDynamicValue((context) => {
    return new device_admin_js_1.SchemaTapConfigurator(null);
});
container.bind(types_1.TYPES.FirmwareUpdateProvider).toDynamicValue((context) => {
    return container.get(types_1.TYPES.S3FirmwareUpdateProvider);
});
container.bind(types_1.TYPES.TapNLinkUpdater).toDynamicValue((context) => {
    return new tapnlink_update_service_1.TapNLinkUpdaterService();
});
container.bind(types_1.TYPES.TapNPassUpdater).toDynamicValue((context) => {
    return new tapnpass_update_service_1.TapnPassUpdater();
});
//# sourceMappingURL=inversify.config.js.map