"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const typescript_rest_1 = require("typescript-rest");
const bodyParser = require("body-parser");
const api_1 = require("./api");
const response_error_1 = require("@iotize/device-client.js/client/api/response/response-error");
const port = 3011;
function getEnv(name) {
    let value = process.env[name];
    if (value == null) {
        throw new Error(`Missing env name: ${name}`);
    }
    return value;
}
let app = express();
app.use(bodyParser.json());
app.use(bodyParser.text());
// app.use(bodyParser.raw())
typescript_rest_1.Server.buildServices(app, api_1.TapRestService, api_1.FirmwareUpdateService);
app.use((err, req, res, next) => {
    if (err instanceof response_error_1.ResponseError || err.response) {
        let statusCode = 400;
        res.set("Content-Type", "application/json");
        res.status(400);
        res.json({ error: err.message, code: statusCode, tapCode: err.response.codeRet() });
    }
    else {
        next(err);
    }
});
app.use((err, req, res, next) => {
    let statusCode;
    if (err instanceof typescript_rest_1.HttpError) {
        if (res.headersSent) { // important to allow default error handler to close connection if headers already sent
            return next(err);
        }
        statusCode = err.statusCode;
    }
    else {
        statusCode = 500;
    }
    res.set("Content-Type", "application/json");
    res.status(statusCode);
    res.json({ error: `Uncaught error. Cause: ${err.message}`, statusCode: statusCode });
});
app.listen(port, function () {
    console.log(`Rest Server listening on port ${port}!`);
});
//# sourceMappingURL=main.js.map