"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios = require("axios");
const core_1 = require("@iotize/device-client.js/core");
class RestFirmwareUpdateProvider {
    constructor(config) {
        this.config = config;
        this.api = axios.default.create(config);
    }
    getLatestVersion(sn) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = `/device/${sn}/firmware/info/latest`;
            let httpResponse = yield this.api.get(url);
            return httpResponse.data.version;
        });
    }
    getForDevice(sn, version) {
        return __awaiter(this, void 0, void 0, function* () {
            let updateUrl = `/device/${sn}/firmware-update/generate/${version}/hex`;
            console.log(`Requesting update at ${updateUrl}`);
            try {
                let httpResponse = yield this.api.get(updateUrl);
                let firmwareUpdateData = core_1.FormatHelper.hexStringToBuffer(httpResponse.data);
                return firmwareUpdateData;
            }
            catch (error) {
                if (error.response) {
                    // The request was made and the server responded with a status code
                    // that falls out of the range of 2xx
                    if (error.response.status == 404) {
                        throw new Error(`Firmware update "${version}" is not available for device "${sn}"`);
                    }
                }
                throw error;
            }
        });
    }
}
exports.RestFirmwareUpdateProvider = RestFirmwareUpdateProvider;
// var getRawBody = require('raw-body');
class S3FirmwareUpdateProvider {
    constructor(config) {
        this.config = config;
        let axioConfig = {
            baseURL: `http://${config.bucket}.s3.dualstack.${config.region}.amazonaws.com`,
            timeout: 10000
        };
        this.api = axios.default.create(axioConfig);
    }
    getLatestVersion(sn) {
        return Promise.reject(new Error("Method not implemented."));
    }
    getForDevice(sn, version) {
        return __awaiter(this, void 0, void 0, function* () {
            let updateUrl = `/tap-firmware-update/${sn}/${version}.izsp`;
            console.log(`Requesting update at ${updateUrl}`);
            try {
                let httpResponse = yield this.api.get(updateUrl, {
                    responseType: 'arraybuffer'
                });
                let firmwareUpdateData = Buffer.from(httpResponse.data, 'binary');
                return firmwareUpdateData;
            }
            catch (error) {
                if (error.response) {
                    // The request was made and the server responded with a status code
                    // that falls out of the range of 2xx
                    if (error.response.status == 403) {
                        throw new Error(`Firmware update "${version}" is not available for device "${sn}"`);
                    }
                }
                throw error;
            }
        });
    }
}
exports.S3FirmwareUpdateProvider = S3FirmwareUpdateProvider;
//# sourceMappingURL=index.js.map