"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const readline = require("readline-sync");
const Colors = require("colors");
class Input {
    constructor() {
    }
    confirm(message = 'Are you sure to continue ?', defaultValue) {
        let choices = ['yes', 'y', 'n', 'no'];
        message += ' (' + choices.join('/') + ')';
        let result = this.readWhileCondition(message, defaultValue, (input) => {
            if (!input) {
                input = '';
            }
            input = input.trim();
            return choices.indexOf(input) !== -1;
        });
        return ['yes', 'y'].indexOf(result) > -1;
    }
    read(question) {
        return this.readWhileCondition(question);
    }
    readWhileCondition(question, defaultValue, validator) {
        let validatorFct = undefined;
        if (validator) {
            if (typeof validator === 'function') {
                validatorFct = validator;
            }
            else {
                validatorFct = (validator.validate).bind(validator);
            }
        }
        if (!validatorFct) {
            validatorFct = (input) => {
                return true;
            };
        }
        let input;
        question = Colors.bgBlue(Colors.white(Colors.bold(question)));
        let prompt = "";
        if (defaultValue) {
            prompt += " [" + defaultValue + "] ";
        }
        prompt += " > ";
        do {
            process.stdout.write(question);
            input = readline.question(prompt, {
                defaultInput: defaultValue,
            });
        } while (!validatorFct(input));
        return input;
    }
}
exports.Input = Input;
//# sourceMappingURL=input.js.map