"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const impl_1 = require("@iotize/device-client.js/protocol/impl");
const api_1 = require("@iotize/device-client.js/client/api");
const lib_1 = require("@iotize/device-com-ble.node/lib");
const inversify_config_1 = require("../../inversify.config");
const types_1 = require("../../types");
const error_detection_1 = require("@iotize/device-client.js/core/error-detection");
require("@iotize/device-admin.js/extensions");
const MAX_NOBLE_SIZE = 144;
/**
 * TODO move to lib
 */
class TapNLinkUpdaterService {
    constructor(fimwareUpdateApi = inversify_config_1.container.get(types_1.TYPES.FirmwareUpdateProvider)) {
        this.fimwareUpdateApi = fimwareUpdateApi;
    }
    update(tap, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tap.protocol instanceof impl_1.AbstractComProtocol) {
                tap.protocol.options.send.timeout = Math.max(10000, tap.protocol.options.send.timeout);
            }
            let sn = (yield tap.service.device.getSerialNumber()).body();
            let currentVersion = (yield tap.service.device.getFirmwareVersion()).body();
            global.output.infonl(`Firmware version: ${currentVersion}`);
            if (options.version == 'latest') {
                try {
                    options.version = yield this.fimwareUpdateApi.getLatestVersion(sn);
                }
                catch (err) {
                    console.debug(err);
                    console.info(`Cannot get version corresponding to "latest". ${err.message}`);
                }
            }
            if (currentVersion.startsWith(`${options.version} `)) {
                global.output.warn(`Tap firmware has already version ${options.version}`);
                if (!options.force) {
                    return;
                }
            }
            else if (currentVersion >= options.version) {
                if (!options.force && !global.input.confirm(`Are you sure to downgrade tap version from "${currentVersion}" to "${options.version}" ?`)) {
                    return;
                }
            }
            if (options.factoryReset) {
                global.output.infonl(`Clearing configuration...`);
                yield tap.factoryReset();
            }
            global.output.infonl(`Requesting firmware update version ${options.version} for ${sn}...`);
            let firmwareUpdateData = yield this.fimwareUpdateApi.getForDevice(sn, options.version);
            try {
                // TODO move check from here
                let expectedCrc = error_detection_1.CRC.fromBytes(firmwareUpdateData); // TODO inject crc compute
                global.output.infonl(`Firmware update date is available.`);
            }
            catch (err) {
                return Promise.reject(err);
            }
            var bar = undefined;
            let maxSinglePacketSize = 0;
            if (!options.chunkSize) {
                if (tap.protocol instanceof lib_1.NobleBLEAdapter) {
                    maxSinglePacketSize = MAX_NOBLE_SIZE;
                }
            }
            else {
                maxSinglePacketSize = options.chunkSize;
            }
            if (!maxSinglePacketSize) {
                maxSinglePacketSize = api_1.SinglePacket.MAX_PART_SIZE;
            }
            if (tap.protocol instanceof lib_1.NobleBLEAdapter) {
                // Due to error LIBUSB_TRANSFER_STALL with noble when giving more that 177 + 12 bytes
                if (maxSinglePacketSize > MAX_NOBLE_SIZE) {
                    global.input.confirm(`Using packet size too big for BLE ${maxSinglePacketSize}. Max size: ${MAX_NOBLE_SIZE}. Are you sure to continue ?`);
                }
                // maxSinglePacketSize = 160
            }
            // console.info(`Using packet size of ${maxSinglePacketSize} bytes`)
            // let packetBuilder = new SinglePacket.Splitter(maxSinglePacketSize);
            // let packets = packetBuilder.chunk(firmwareUpdateData);
            // if (!tap.encryption) {
            // await tap.enableEncryption(true)
            // }
            let fwUpdated = tap.firmwareUpdater({
                maxSinglePacketSize: maxSinglePacketSize
            });
            let observable = fwUpdated.start(firmwareUpdateData);
            observable.subscribe({
                next: (event) => {
                    switch (event.step) {
                        case 'load':
                            let progress = event.payload;
                            if (!bar) {
                                bar = global.output.progressBar('Uploading to device [:bar] :rate chunk/seconds :percent :etas (:current/:total)', {
                                    total: progress.total
                                });
                            }
                            else {
                                bar.tick();
                            }
                            break;
                        case 'apply':
                            global.output.infonl(`Applying update...`);
                        // case 'finish':
                        //     global.output.infonl(`Update done!`);
                    }
                },
                complete: () => {
                },
                error: (err) => {
                    console.info(err);
                    global.output.error(err);
                }
            });
            yield observable.toPromise();
        });
    }
}
exports.TapNLinkUpdaterService = TapNLinkUpdaterService;
//# sourceMappingURL=tapnlink-update.service.js.map