"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const device_com_websocket_js_1 = require("@iotize/device-com-websocket.js");
const lib_1 = require("@iotize/device-com-ble.node/lib");
const URL = require("url");
const device_com_socket_node_1 = require("@iotize/device-com-socket.node");
const device_com_mock_js_1 = require("@iotize/device-com-mock.js");
const operators_1 = require("rxjs/operators");
const inversify_config_1 = require("../inversify.config");
const types_1 = require("../types");
/**
 * @deprecated
 * Use lib instead
 */
class ProtocolFactory {
    static fromUrl(url) {
        let aliases = global.getConfig('aliases', {});
        // console.log('Aliases: ', aliases);
        if (url in aliases) {
            url = aliases[url];
        }
        if (!url || url.length === 0) {
            throw new Error(`Protocol identifier must not be null`);
        }
        console.log(`Creating protool from url: "${url}"`);
        if (url.startsWith("ble:")) {
            let bleAddress = url.substr(4);
            // global.output.info("BLE @ = " + bleAddress);
            return ProtocolFactory.create("ble", {
                address: bleAddress.toLowerCase()
            });
        }
        else {
            var parsedUrl = URL.parse(url);
            if (!parsedUrl.protocol) {
                //throw new Error(`Cannot find protocol from url ${url}. Make sure it's a valid one!`);
                return ProtocolFactory.fromDeviceName(url);
            }
            // global.output.info(`Using protocol ${parsedUrl.protocol}`);
            return ProtocolFactory.create(parsedUrl.protocol, {
                url: url
            });
        }
    }
    /**
     * Try to create protocol from device name
     */
    static fromDeviceName(url, scanDuration = 5000) {
        return inversify_config_1.container
            .get(types_1.TYPES.DeviceScannerProvider)()
            .then((scanner) => {
            return scanner
                .startScan({
                protocols: {
                    'WIFI': true,
                    'BLE': true
                },
                duration: scanDuration
            })
                .pipe(operators_1.filter((entry) => {
                console.log('Check filter on device ', entry.name || 'UNAMED', entry.address);
                return entry.name ? entry.name.endsWith(url) : false;
            }))
                .pipe(operators_1.map((entry) => {
                // console.info('Found device ', entry);
                return ProtocolFactory.fromScanItem(entry);
            }))
                .pipe(operators_1.first())
                .toPromise();
        });
    }
    static fromScanItem(entry) {
        switch (entry.type) {
            case 'BLE':
                return new lib_1.NobleBLEAdapter(entry.payload);
            case 'WIFI':
                return new device_com_socket_node_1.SocketProtocol({
                    host: entry.address,
                    port: 2000
                });
            default:
                throw new Error(`Invalid scan item type: ${entry.type}`);
        }
    }
    static create(protocolType, extraOptions) {
        console.log(`Creating protocol ${protocolType}...`, extraOptions);
        if (!extraOptions) {
            extraOptions = {};
        }
        if (!extraOptions.url) {
            extraOptions.url = "ws://localhost:2000";
        }
        switch (protocolType) {
            case 'mock':
                return ProtocolFactory.createMockProtocol(extraOptions);
            case "ble":
                return ProtocolFactory.createBleProtocol(extraOptions);
            case "tcp":
            case "tcp:":
                let protocol = new device_com_socket_node_1.SocketProtocol();
                let parsedOptions = URL.parse(extraOptions.url);
                protocol.setSocketOptions({
                    host: parsedOptions.hostname,
                    port: parseInt(parsedOptions.port)
                });
                return Promise.resolve(protocol); // TODO
            case "websocket":
            case "ws":
            case "wss":
            case "wss:":
            case "ws:":
                return Promise.resolve(new device_com_websocket_js_1.WebSocketProtocol({
                    url: extraOptions.url
                }));
            default:
                return Promise.reject(new Error(`Not implemented protocol: ${protocolType}`));
        }
    }
    static createMockProtocol(extraOptions) {
        return Promise.resolve(new device_com_mock_js_1.MockProtocol()); // TODO
    }
    static createBleProtocol(options) {
        if (!options) {
            throw new Error("BLE device address is required");
        }
        return inversify_config_1.container
            .get(types_1.TYPES.NobleProvider)()
            .then((noble) => {
            var scanner = new lib_1.NobleBleScanner(noble);
            return scanner
                .getDeviceByAddress(options.address, {
                timeout: ProtocolFactory.BLE_SCAN_TIMEOUT
            })
                .then((bleDevice) => {
                scanner.stop();
                let ble = new lib_1.NobleBLEAdapter(bleDevice);
                ble.options.send.timeout = 20000;
                return ble; // TODO fix. (test wont pass otherwise)
            });
        });
    }
}
ProtocolFactory.BLE_SCAN_TIMEOUT = 5000;
exports.ProtocolFactory = ProtocolFactory;
//# sourceMappingURL=protocol-factory.js.map