"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const impl_1 = require("@iotize/device-client.js/relay/impl");
const relay_1 = require("@iotize/device-com-websocket.js/relay");
const relay_2 = require("@iotize/device-com-socket.node/dist/relay");
const web_socket_protocol_1 = require("@iotize/device-com-websocket.js/web-socket-protocol");
const device_com_socket_node_1 = require("@iotize/device-com-socket.node");
const url_1 = require("url");
class RelayServerFactory {
    static fromProtocol(comProtocol, options) {
        let serverType = '';
        if (comProtocol instanceof web_socket_protocol_1.WebSocketProtocol) {
            serverType = 'websocket';
            if (!options) {
                options = {
                    port: parseInt(url_1.parse(comProtocol.params.url).port)
                };
            }
        }
        else if (comProtocol instanceof device_com_socket_node_1.SocketProtocol) {
            serverType = 'socket';
            if (!options) {
                options = {
                    port: comProtocol.socketOptions.port
                };
            }
        }
        else {
            throw new Error(`Not relay server factory from this protocol`);
        }
        return RelayServerFactory.fromType(serverType, options);
    }
    static fromType(type, options) {
        let relayServer;
        switch (type) {
            case 'websocket':
                relayServer = new impl_1.RelayServer(new relay_1.WebSocketAdapter.Server(), options);
                break;
            case "socket":
                relayServer = new impl_1.RelayServer(new relay_2.SocketAdapter.Server(), (options));
                break;
            default:
                throw new Error(`Server type ${type} not implemented yet`);
        }
        return relayServer;
    }
}
exports.RelayServerFactory = RelayServerFactory;
//# sourceMappingURL=relay-server-factory.js.map